/*---- Includes -------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

#include "nrf_drv_i2s.h"
#include "nrf_gpio.h"
#include "uc_config.h"
#include "uiSound.h"


/*---- Definitions ----------------------------------------------------------*/

#define SDMODE_PIN       14
#define TX_BUFFER_SIZE    8                    /* Length of transmit buffer, */
                                               /* specified in 32-bit words  */

/*---- Global Data ----------------------------------------------------------*/

static ASoundCompleteCallback  mCallback;    /* The event handler (callback) */

static ATune  mCurrent;
static bool   mTuneComplete;

static unsigned long *mTuneTx;
static unsigned long *mTuneStop;
static uint32_t       mTxBuffer[TX_BUFFER_SIZE];

static const unsigned long  ourLovelyJingle[] = {
0x7FFF7FFF,
0x9F189F18,
0xBC55BC55,
0xD5F3D5F3,
0xEA6BEA6B,
0xF882F882,
0xFF60FF60,
0xFE9BFE9B,
0xF63FF63F,
0xE6CDE6CD,
0xD132D132,
0xB6B8B6B8,
0x98F798F7,
0x79B779B7,
0x5AD75AD7,
0x3E313E31,
0x257D257D,
0x12351235,
0x05820582,
0x00270027,
0x02750275,
0x0C490C49,
0x1D0D1D0D,
0x33BF33BF,
0x4F034F03,
0x6D376D37,
0x8C8A8C8A,
0xAB1DAB1D,
0xC71BC71B,
0xDED5DED5,
0xF0E0F0E0,
0xFC28FC28,
0xFFFEFFFE,
0xFC28FC28,
0xF0E0F0E0,
0xDED5DED5,
0xC71BC71B,
0xAB1DAB1D,
0x8C8A8C8A,
0x6D376D37,
0x4F034F03,
0x33BF33BF,
0x1D0D1D0D,
0x0C490C49,
0x02750275,
0x00270027,
0x05820582,
0x12351235,
0x257D257D,
0x3E313E31,
0x5AD75AD7,
0x79B779B7,
0x98F798F7,
0xB6B8B6B8,
0xD132D132,
0xE6CDE6CD,
0xF63FF63F,
0xFE9BFE9B,
0xFF60FF60,
0xF882F882,
0xEA6BEA6B,
0xD5F3D5F3,
0xBC55BC55,
0x9F189F18,
0x7FFF7FFF,
0x60E560E5,
0x43A843A8,
0x2A0A2A0A,
0x15921592,
0x077B077B,
0x009D009D,
0x01620162,
0x09BE09BE,
0x19301930,
0x2ECB2ECB,
0x49454945,
0x67066706,
0x86468646,
0xA526A526,
0xC1CCC1CC,
0xDA80DA80,
0xEDC8EDC8,
0xFA7BFA7B,
0xFFD6FFD6,
0xFD88FD88,
0xF3B4F3B4,
0xE2F0E2F0,
0xCC3ECC3E,
0xB0FAB0FA,
0x92C692C6,
0x73737373,
0x54E054E0,
0x38E238E2,
0x21282128,
0x0F1D0F1D,
0x03D503D5,
0x00000000,
0x03D503D5,
0x0F1D0F1D,
0x21282128,
0x38E238E2,
0x54E054E0,
0x73737373,
0x92C692C6,
0xB0FAB0FA,
0xCC3ECC3E,
0xE2F0E2F0,
0xF3B4F3B4,
0xFD88FD88,
0xFFD6FFD6,
0xFA7BFA7B,
0xEDC8EDC8,
0xDA80DA80,
0xC1CCC1CC,
0xA526A526,
0x86468646,
0x67066706,
0x49454945,
0x2ECB2ECB,
0x19301930,
0x09BE09BE,
0x01620162,
0x009D009D,
0x077B077B,
0x15921592,
0x2A0A2A0A,
0x43A843A8,
0x60E560E5
};

/*---- Prototypes -----------------------------------------------------------*/

static void  fireSoundComplete (void);

static void onTxPrepare (uint32_t const *p_data_received,
                         uint32_t       *p_data_to_send,
                         uint16_t        number_of_words);


/*---- Implementation -------------------------------------------------------*/

/*.... Public Functions .....................................................*/

void  soundInitialise (ASoundCompleteCallback pCB)
{
    nrf_drv_i2s_config_t  config = NRF_DRV_I2S_DEFAULT_CONFIG;


    mCallback     = pCB;
    mTuneComplete = false;

    nrf_drv_i2s_init    (&config, onTxPrepare);
    nrf_gpio_pin_clear  (SDMODE_PIN);
    nrf_gpio_cfg_output (SDMODE_PIN);
}


void  soundUpdate (void)
{
    if (mTuneComplete)
    {
        fireSoundComplete ();
        mTuneComplete = false;
    }
}


void  soundPlay (ATune tune)
{
    int  ret;

    if (tune != tnInformation)
        return;

    mCurrent      = tune;
    mTuneComplete = false;

    mTuneTx   = (unsigned long *)&(ourLovelyJingle[0]);
    mTuneStop = mTuneTx + sizeof (ourLovelyJingle) / sizeof (long);

    nrf_gpio_pin_set  (SDMODE_PIN);

//    ret = nrf_drv_i2s_start (NULL, mTxBuffer, TX_BUFFER_SIZE, 0);

//    if (ret != NRF_SUCCESS)
//        ret = ret;
}


/*.... Private Functions ....................................................*/

static void  fireSoundComplete (void)
{
    ASoundCompleteEvent   sce;


    if (mCallback == NULL)             /* No callback -> cannot report event */
        return;

    sce.sender = 0;
    sce.tune   = mCurrent;

    mCallback (EVENT_SOUND_COMPLETE, (char *)&sce);
}


/*.... Interrupt Service Routines ...........................................*/

static void  onTxPrepare (uint32_t const *p_data_received,
                          uint32_t       *p_data_to_send,
                          uint16_t        number_of_words)
{
    if (p_data_received != NULL)
        number_of_words = 0;

    if (p_data_to_send  != NULL)
    {
        if (mTuneTx >= mTuneStop)
        {
            nrf_drv_i2s_stop   ();
            nrf_gpio_pin_clear (SDMODE_PIN);

            mTuneComplete = true;
            return;
        }

        while (number_of_words)
        {
            if (mTuneTx < mTuneStop)
               *p_data_to_send = *mTuneTx;
            else
               *p_data_to_send = 0;

            number_of_words--;
            mTuneTx++;
        }
    }
}


/*-- END - END - END - END - END - END - END - END - END - END - END - END --*/
